@* Functional minimization. The first thing we need is to be able to minimize a functional
without any constraints. Strictly speaking the correct thing to do would be to define a base
class for functional minimization and specialize it depending on the stop criteria, but in the
real world we face only two of them: $$\eqalign{\vert x_i-x_i^*\vert&\leq\vert
x_i^*\vert\\{reltol}+\\{abstol}\cr \Vert\nabla f(x)\Vert&\leq\\{abstol}.}$$ Of course, the first
condition only makes sense when $\\{abstol}\cdot\\{reltol}=0$. From that observation we decided
to define only one functional minimization function that can take into account all of the above
stopping criteria.

The function we'll define will take various arguments: the first are the functional to be
minimized, the starting point, the line search algorithm and the search direction algorithm. The
starting point will be overwritten with the optimal point approximation and returned on exit. The
next arguments define the stopping criteria: $\\{abstol}$ and~$\\{reltol}$ have the meaning of
that first stopping condition described above. They can be both positive, in which case the
optimization stops when {\it both\/} the absolute error and the relative error conditions are
met. If you don't want one of them to have an effect, simply assign it zero. The~$\\{gtol}$
argument specifies, if nonzero, the stop condition for the gradient norm. At last, the user can
input the maximum allowed number of iterations and a pointer to a function that gets the current
piont at each iteration (possibly for displaying interactively).
@c
/* Empty, waiting for |export|. */
@ @(fmin.h@>=
#ifndef __MATH_FMIN__
#define __MATH_FMIN__ 1.0
#include <math/functionalbase.h>
#include <math/linesearchbase.h>
#include <math/searchdirbase.h>
namespace math
{
    template<class T>@/
    matrix<T,unstructured,dense>&
    fmin(functional::base<T>* f,
         matrix<T,unstructured,dense>& x,
	 linesearch::base<T>* lsearch,searchdir::base<T>* sdir,
	 T abstol=1e-4, T reltol=1e-3, T gtol=1e-3, unsigned long maxiter=1000,
	 void (*disp)(const matrix<T,unstructured,dense>&)=0)
    {
	@<Functional minimization algorithm@>@;
    }
}
#endif
@ The algorithm is pretty simple: we keep calling the line minimization function until the stop
criteria is met or the maximum number of iterations is reached (in which case we return the last
line minimizer). The only trick in this function is that we only use the gradient for line
minimization if~$\\{gtol}>0$. In this case, our function keeps the last function gradient
in~$\\{grad}$ (which is otherwise not used).
@<Functional minimization algorithm@>=
@:functional}{grad@>
@:linesearch}{minimize@>
@:searchdir}{dir@>
{
    matrix<T,unstructured,dense> x0,grad,dir;
    if(gtol) f->grad(x,grad);
    bool stop=false;
    for(unsigned long iter=0;!stop && iter!=maxiter;++iter)
    {
	x0=x;
	if(gtol) x=lsearch->minimize(f,x,grad,sdir->dir(f,x,&dir));
	else x=lsearch->minimize(f,x,sdir->dir(f,x,&dir));
	@<Update functional minimization stop criteria@>@;
	if(disp) disp(x);
    }
    return x;
}
@ We test if all stopping criteria are satisfied. For the relative tolerance we have to check if
it makes sense: if~$x_i=0$ no relative tolerance can be reached (in practice).
@<Update functional minimization stop criteria@>=
@:matrix}{rows@>@:functional}{grad@>
stop=true;
if(abstol)@+
for(index i=1;i<=x.rows() && stop;++i)
    stop = (fabs(x(i)-x0(i))<=abstol);
if(reltol)@+
    for(index i=1;i<=x.rows() && stop;++i)
	if(x(i))
	    stop = (fabs(x(i)-x0(i))<=fabs(x(i))*reltol);
if(gtol)
    stop &= (norm2(f->grad(x,grad))<=gtol);

