@** Functionals. Functionals are the \math{} library representation of functionals. In the
most simple situation, functionals take arguments and return the value of a function at some
point. In other situations, such as optimization algorithms, we may want the gradient and/or
the Hessian of the function at some point. Here we define the basic interface in a base
class from which specific functionals will be derived.
@s functional int
@c
/* Not until |export| */
@ @(functionalbase.h@>=
#ifndef __MATH_FN__
#define __MATH_FN__ 1.00
#include <math/math.h>
#include <math/symmetric.h>
#include <math/sparse.h>
namespace math
{
    @[namespace @!functional@]
    {
	template<class T>@;
	class base
	{
	public:@;
	    @<Functional base class methods@>@;
	};
    }
}
#endif
@ The first thing we do is to declare a virtual destructor, so that we can 
derive classes from the base class.
@<Functional base class methods@>=
virtual ~base(void)@+ {}
@ Before defining the interface we need to consider some points about functionals: first, a
functional should be able to get any kind of matrix as the point, that is, the~$x$ in~$f(x)$
could be sparse, dense and so on. Also, we should be able to create a list containing
various types of functionals. The impossibility of this ideal situation is summarized by
Stroustrup: ``A member template cannot be virtual.'' This is a design decision, and a wrong
one in my point of view. We have then only two possibilities: either we make the |matrix|
class derived, so we can pass pointers, or we fix the type of the parameter. The first
option results in performance degradation, so we're stuck with the second. Fortunately most
functionals take vectors as arguments, so that matrix structures are not a big deal. We will
assume that |dense| vectors are the best compromise.
@<Functional base class methods@>=
T operator()(const matrix<T,unstructured,dense>& x) @+ {@+ return eval(x);@+ }
virtual T eval(const matrix<T,unstructured,dense>& x) = 0;
@ If possible, a functional should compute the gradient at a point. Here we face another
interface decision: functionals are supposed to be used to compute lots of points, that is,
when you create a functional, usually you want to evaluate it in a set of points, not just
only one. If a evaluation method created storage for the result each time it was called,
then we would be facing a serious performance threat. For this reason we require the user to
pass the result variable as an argument. This requirement, in turn, makes the type of the
result matrix fixed for the same reasons discussed above. The method returns the result
(that is, |dest|), which is a useful behavior in certain applications, as for example in
function compositions like~$f(g(x))$. The size of the |dest| matrix at input {\em should be
checked by the method}. This will be normally performed via the |matrix::init| method. Since 
this method checks for same dimensions, the performance does not suffer too much.
@<Functional base class methods@>=@!@:functional::base}{grad@>
virtual matrix<T,unstructured,dense>&
grad(const matrix<T,unstructured,dense>& x, matrix<T,unstructured,dense>& dest) = 0;
@ The same thing is valid for the Hessian, except here we require the Hessian to be
symmetric (which it is by definition). The same remarks about |dest| size made above are
valid here. We provide two methods, one for |sparse| matrices (for large optimization problems it 
may be crucial to get a sparse version).
@<Functional base class methods@>=@!@:functional::base}{hess@>
virtual matrix<T,symmetric,dense>&
hess(const matrix<T,unstructured,dense>& x, matrix<T,symmetric,dense>& dest) = 0;
virtual matrix<T,symmetric,sparse>&
hess(const matrix<T,unstructured,dense>& x, matrix<T,symmetric,sparse>& dest) = 0;
