@* Backtracking. This line search algorithm uses the gradient information. We begin with a unit
step~$\lambda$, and until we have $$f(x_0+\lambda\cdot\\{dir})\leq f(x_0)+\lambda\alpha\nabla
f(x_0)^T\\{dir},$$ we update the step with~$\beta\lambda$. The algorithm parameters are~$\alpha$
and~$\beta$, where~$0<\alpha<0.5$ and~$0<\beta<1$.
@c /* Empty, waiting for |export|. */
@ @(linesearch/backtracking.h@>=
#ifndef __MATH_BACKTRACKING_LINESEARCH__
#define __MATH_BACKTRACKING_LINESEARCH__ 1.0
const double backtracking_infty=HUGE_VAL; /* Some compilers have problems using |HUGE_VAL|. */
#include<math/algebra.h>
#include<math/linesearchbase.h>
namespace math
{
    @[namespace linesearch@]
    {
	template<class T>@;
	class @[backtracking : public base<T>@]
	{
	    double alpha,beta;
	    unsigned long maxiter;
	public:
	    ~backtracking(void){}
	    @<Backtracking line search methods@>@;
	};
    }
}
#endif
@ First we provide a constructor that enable us to set all parameters at creation time, with some 
defaults in case we don't care about it.
@<Backtracking line search methods@>= 
backtracking(double a=0.3, double b=0.8,unsigned long m=1000)
: alpha(a), beta(b), maxiter(m) @+{@+}
@ The first thing we do is to define the method that doesn't take the gradient as an argument.
Since we need the gradient, what this method does is to compute it and call the correct method.
@<Backtracking line search methods@>=@!@:backtracking}{minimize@>
matrix<T,unstructured,dense>
minimize(functional::base<T>* f,const matrix<T,unstructured,dense>& x0, const
	 matrix<T,unstructured,dense>& dir)
{
    matrix<T,unstructured,dense> g0;
    g0=f->grad(x0,g0);@:functional}{grad@>
    return minimize(f,x0,g0,dir);
}
@ 
@<Backtracking line search methods@>=@!@:backtracking}{minimize@>
matrix<T,unstructured,dense>
minimize(functional::base<T>* func,const matrix<T,unstructured,dense>& x0,
	 const matrix<T,unstructured,dense>& g0,
	 const matrix<T,unstructured,dense>& dir)
{
    matrix<T,unstructured,dense> x;
    double ftreshold=func->eval(x0);@:functional}{eval@>
    double gtreshold=alpha*dot(g0,dir);
    double stepsize=1/beta;
    unsigned long iter=0;
    double fval;
#ifdef __MATH_DEBUG__
cout << "[math]: backtracking line search begin.\n";
cout << "[math]: x0 = ";
for(index i=1;i<=x0.rows();++i)
    cout << x0(i) << ' ';
cout << '\n';
cout << "[math]: dir = ";
for(index i=1;i<=dir.rows();++i)
    cout << dir(i) << ' ';
cout << '\n';
#endif
    do
    {
	@<Compute next function value for |backtracking|@>;
	if(++iter==maxiter) throw error::maxiterations();
    }
    while(fval>ftreshold+stepsize*gtreshold);
#ifdef __MATH_DEBUG__
cout << "[math]: backtracking line search end.\n";
#endif
    return x;
}
@ @<Compute next function value for |backtracking|@>=
fval=backtracking_infty;
while(!finite(fval))
{
    x=dir;@:matrix}{=@>
    x*=(stepsize*=beta);@:matrix}{*=@>
    x+=x0;@:matrix}{+=@>
#ifdef __MATH_DEBUG__
cout << "[math]: x = ";
for(index i=1;i<=x.rows();++i)
    cout << x(i) << ' ';
cout << '\n';
#endif
    try { fval=func->eval(x); }@:functional}{eval@>
    catch(error::domain e) { fval=backtracking_infty; }
#ifdef __MATH_DEBUG__
cout << "[math]: f(x)=" << fval << ", stepsize=" << stepsize << 
    ", treshold=" << ftreshold+stepsize*gtreshold << '\n';
#endif
    if(stepsize==0.0) fval=ftreshold+stepsize*gtreshold;
}
