@** The LU decomposition. We now begin to build the backbone of any linear algebra library:
the matrix decomposition functions. We begin with the most simple, the LU decomposition. The
goal is to decompose a matrix~$A$ into a product of a lower-triangular matrix~$L$ and an
upper-triangular matrix~$U$, that is, we want to have~$A=LU$. The main purpose of this
decomposition is to solve linear systems: if we want to solve~$Ax=b$, we decompose
matrix~$A$ and then solve two systems, $Ly=b$ and~$Ux=y$ to find the solution. The point is
that solving triangular systems is very easy by back or forward substitution.
@c
/* Empty, waiting for |export| */
@ @(lu.h@>=
#ifndef __MATH_LU__
#define __MATH_LU__ 1.0
#include <math/math.h>
#include <vector>
#include <complex> /* For templatized |abs|. */
namespace math
{@/
    namespace lu
    {
    @<LU prototypes@>@;
    }
}
#endif
@ We can understand the way the decomposition works by considering the first step in
a~$3\times3$ matrix. We build a matrix~$M_1$ such that the first column of~$M_1 A$ is zero,
except for the first element. The construction is
$$ M_1 A = 
\bmatrix{ 1 & 0 & 0 \cr
-{a_{21}\over a_{11}}&1&0 \cr
-{a_{31}\over a_{11}}&0&1}
\bmatrix{a_{11}&a_{12}&a_{13} \cr
a_{21}&a_{22}&a_{23} \cr
a_{31}&a_{32}&a_{33}}
=
\bmatrix{a_{11}&a_{12}&a_{13} \cr 
0& a_{22}-{a_{21}a_{12}\over a_{11}}    &a_{23}-{a_{21}a_{13}\over a_{11}} \cr
0&a_{32}-{a_{31}a_{12}\over a_{11}}&a_{33}-{a_{31}a_{13}\over a_{11}}}.
$$
Now we can repeat the procedure with the remaining blocks of the matrix so that
$$ M_2 M_1 A = U, $$ where~$U$ is upper-triangular. Since~$M_i$ is lower-triangular, so it
is its inverse, and defining~$L=M_1^{-1} M_2^{-1}$ we finally have~$A=LU$. A matrix of the
form~$M_i$ is called a {\em Gauss transformation}, and the~$i$th column
of~$M_i$ is represented by~$m_i=(0,0,\ldots,1,-\tau_{i+1},\ldots,-\tau_{n})$. The
vector~$\tau=(\tau_{i+1},\ldots,\tau_n)$ is called the {\em Gauss vector}. In this way, a
Gauss transformation~$M_i$ can be represented as~$M_i=(I-\tau^T e_i)$.

There is only one possible problem with the algorithm. Looking at the first step of
the~$3\times 3$ decomposition example we can see that, if~$a_{11}$ is small, we have
numerical problems because the Gauss vector could have too large elements. This effect can
propagate through the rest of the algorithm. The solution is to find the decomposition for a
permuted version of~$A$, such that at every step we have the largest possible denominator in
the Gauss vector. The denominator is called the {\em pivot}, and the method of permutations
is called {\em pivoting}. The LU decomposition has a workload of~$O(n^3)$ flops (actually
exactly $2n^3/3$) and, if with pivoting, $O(n^2)$ comparisons.
@ Our basic function will be one that will overwrite the input matrix with~$U$ on the upper
part and~$L$ on the lower part (it can be easily shown -- see Golub and Van Loan -- that~$L$ 
is a row permutation of the Gauss vectors). A vector |pivots| will store the data about
the permutations, such that at step~$k$ we multiply~$A$ by the identity matrix with rows~$k$ 
and |pivots(k)| swapped. The function returns~$1$ if the number of permutations is even
and~$-1$ otherwise. Since we will overwrite the matrix at will, we require it to have no
structure.
@<LU prototypes@>=
template<class T, template<@[class@]> class storage>@/
int decompose(matrix<T,unstructured,storage>* A, vector<index>* pivots)
{
    int permutations=1; /* No permutations yet. */

    index n=A->rows();
    if(n!=A->cols()) throw error::nonsquare();

    pivots->resize(n-1,0); /* Reserve space for permutation data. */

    for(index k=1;k!=n;++k)
    {
        T pivot;
        @<Search for pivot and swap rows if necessary@>@;
        if(pivot!=0)
            @<Apply Gauss transformation@>@;
    }
    return permutations;
}
@ @<Search for pivot...@>=
pivot=A->get(k,k);
(*pivots)[k-1]=k; /* STL vector has base index 0. */
for(index i=k+1;i<=n;++i)
    if(abs(A->get(i,k))>abs(pivot))
{
    pivot=A->get(i,k);
    (*pivots)[k-1]=i;
}
if((*pivots)[k-1]!=k) /* If need to swap. */
{
    permutations*= -1;
    A->swaprows(k,(*pivots)[k-1]);
}
@ @<Apply Gauss transformation@>=
for(index i=k+1;i<=n;++i)
{    
    if(!finite(A->entry(i,k)=A->get(i,k)/A->get(k,k))) throw error::singular();
    for(index j=k+1;j<=n;++j) A->entry(i,j)=A->get(i,j)-A->get(i,k)*A->get(k,j);
}
@* Solving a linear system. As hinted before, the principal application of the LU
decomposition is the solution of linear systems. The function we will define will solve a
matrix linear system~$AX=B$. The decomposition just defined gives us information to compute
$P_{n-1}\ldots P_1 A = LU$, where~$P_i$ are permutation matrices defined by the
|pivots| vector. Having the LU decomposition, we solve the system~$Ax=b$ by first solving
$Ly= P_{n-1} \ldots P_1 b$ and then $Ux=y$. By doing this to all columns of~$X$
and~$B$ we are able to solve~$AX=B$. We define now a function that, given a decomposition
and a matrix~$B$, overwrite in~$B$ the solution to~$AX=B$. We define this function
separately because if you want to solve~$A^kX=B$, you need to perform only one decomposition
and call this function~$k$ times.
@<LU prototypes@>=
template<class T, template<@[class@]> class storage>@/
void finish(const matrix<T,unstructured,storage>& A, const vector<index>& pivots,
           matrix<T,unstructured,storage>* B)
{
    index n=A.rows();
    
    for(index i=1;i<n;++i)     /* $B \leftarrow P_{n-1} \ldots P_1 B$ */
        B->swaprows(i,pivots[i-1]);

    for(index k=B->cols();k!=0;--k)
    {
        for(index i=1;i<=n;++i) /* Solve $Ly=B(:,k)$ */
        {
            T inner=0;
            for(index j=1;j!=i;++j)
                inner += A(i,j)*B->get(j,k);
            B->entry(i,k)=B->get(i,k)-inner;
        }
        for(index i=n;i>=1;--i) /* Solve $Ux=B(:,k)$. */
        {
            T inner=0;
            for(index j=i+1;j<=n;++j)
                inner += A(i,j)*B->get(j,k);
            if(!finite(B->entry(i,k)=(B->get(i,k)-inner)/A(i,i)))
		throw error::singular();
        }
    }
}
@ Finally, we provide an interface (that will destroy the original matrix, by the way). Note 
that we can compute the inverse of a matrix by calling this function with~$B=I_n$.
@<LU prototypes@>=
template<class T, template<@[class@]> class storage>@/
matrix<T,unstructured,storage>&
solve(matrix<T,unstructured,storage>* A, matrix<T,unstructured,storage>* B)
{
    vector<index> pivots;
    decompose(A,&pivots);
    finish(*A,pivots,&B);
    return *B;
}
@* Interfacing with \lapack. The routines just written enable you to compute the LU
decomposition of any type of matrix. For certain types, however, we have extremely efficient 
decomposition functions already coded in the \lapack{} package. It only makes sense to call
these routines when possible, and the \math{} library provides the ideal transparent
interface: you call the exact same function, and if there is a \lapack{} function to do the
job, then the function will be called. 
@<LU prototypes@>=
#ifdef LAPACK
    @<LU lapack interface@>@;
#endif
@  The key for the transparent operation is specialization. The trick here is that we need
to make sure that the matrix does not share the representation, otherwise the call to the
\lapack{} routine will modify all matrices sharing it.
@<LU lapack interface@>=
template<>@/
int decompose(matrix<fortran::double_precision,unstructured,dense>* A, vector<index>* pivots)
{
    @<Prepare for lapack LU@>@;
    dgetrf(&m,&n,A->storg()->memory(),&m,(int*)pivots->begin(),&status);
    @<Check for lapack LU errors@>@;
    @<Compute number of permutations and store it in |status|@>@;
    return status;
}
@ @<Prepare for lapack LU@>=
fortran::integer m=fortran::integer(A->rows());
fortran::integer n=fortran::integer(A->cols());
fortran::integer status=0;

pivots->resize(min(m,n),0);

@ @<Check for lapack LU errors@>=    
if(status>0) throw error::singular();
if(status<0) throw error::generic();
@ In order to be compatible with our own LU decomposition routine, we need to compute the
number of permutations (1~means even number, $-1$ means odd).
@<Compute number of permutations and store it in |status|@>=
status=1;
for(index i=1;i<(index)min(m,n);i++)
    if((*pivots)[i-1]!=i) status*=-1;
@ @<LU lapack interface@>=
template<>@/
int decompose(matrix<fortran::real,unstructured,dense>* A, vector<index>* pivots)
{
    @<Prepare for lapack LU@>@;
    sgetrf(&m,&n,A->storg()->memory(),&m,(int*)pivots->begin(),&status);
    @<Check for lapack LU errors@>@;
    @<Compute number of permutations and store it in |status|@>@;
    return status;
}
